/**   
 * @Package     : com.ccb.mobile.platform
 * @Copyright   : @版权所有 中国建设银行版权所有
 * @Description : TODO(用一句话描述该文件做什么)
 * Administrator   
 * 2014-4-16
 * @version     : v1.0   
 */

package com.ccb.mobile.platform;

import java.util.HashMap;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;

import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;

import com.ccb.mobile.platform.exception.NetworkException;
import com.ccb.mobile.platform.exception.ParseDataException;
import com.ccb.mobile.platform.exception.RequestParametersErrorException;
import com.ccb.mobile.platform.parse.json.JsonHelper;
import com.ccb.mobile.platform.utils.MbsLogManager;

/**
 * 平台控制类,处理逻辑层和展现层的交互
 * 
 * @ClassName : PlatformController
 * @author : LiangYJ
 * @date : 2014-4-16
 * 
 */
public class PlatformController {
  private static final int MESSAGE_POST_RESULT = 0x1;// 回调标志
  private static final int MESSAGE_POST_PROGRESS = 0x2;// 处理过程标志
  private static final int MESSAGE_POST_ERROR = 0x3;// 参数错误标志

  private static Handler mHandler;

  /**
   * 
   * 交易调用
   * 
   * @param context
   *          上下文
   * @param params
   *          交易参数
   * @param callback
   *          交易回调
   * @return void: 返回类型
   */
  public static void callInterface(Context context, HashMap<String, String> params, String channel_url, Callback callback) {
    MbsLogManager.logD("#PlatformController# callInterface");
    // if(!checkUIThread()){
    // throw new UIThreadException(Constants.EXCEPTION_NOT_UI_THREAD_DETAIL,
    // new Throwable(Constants.EXCEPTION_NOT_UI_THREAD));
    // }
    if (context == null || params == null || params.isEmpty()) {
      throw new RequestParametersErrorException(Constants.EXCEPTION_REQUEST_PARAMS_ERROR_DETAIL, new Throwable(Constants.EXCEPTION_REQUEST_PARAMS_ERROR_DETAIL));
    }

    mHandler = new Handler() {
      @Override
      public void handleMessage(android.os.Message msg) {
        Result<?> result = (Result<?>) msg.obj;
        switch (msg.what) {
        case MESSAGE_POST_RESULT: {
          BackgroundThread thread = result.mThread;
          if (thread.mCallback != null) {
            thread.mCallback.onFinished(thread.isSuccess(), result.mData[0]);
            // thread.interrupt();
          }
          break;
        }
        case MESSAGE_POST_PROGRESS: {
          break;
        }
        case MESSAGE_POST_ERROR: {
          BackgroundThread thread = result.mThread;
          if (thread.mCallback != null) {
            thread.mCallback.onFinished(thread.isSuccess(), thread.mErrorMsg);
          }
          // thread.interrupt();
          break;
        }

        default:
          break;
        }
      };
    };

    BackgroundThread thread = new BackgroundThread(context, params, channel_url, callback);
    thread.start();
  }

  public static void callInterface(Context context, String json, String channel_url, Callback callback) {
    HashMap<String, String> params = null;
    try {
      JSONObject object = new JSONObject(json);
      HashMap<String, Object> map = JsonHelper.toMap(object);
      params = new HashMap<String, String>(map.size());
      for (String key : map.keySet()) {
        params.put(key, map.get(key).toString());
      }
      callInterface(context, params, channel_url, callback);
    } catch (JSONException e) {
      e.printStackTrace();
      throw new RequestParametersErrorException(Constants.EXCEPTION_REQUEST_PARAMS_FORM_ERROR, new Throwable(Constants.EXCEPTION_REQUEST_PARAMS_FORM_ERROR));
    } finally {

    }
  }

  public static String getCookies(Context context, String sys) {

    return PlatformManager.getInstance().getCookies(context, sys);
  }

  /**
   * 断点续传
   * @author dongguangyu
   * params采用键值对形式：
   * 必须包含参数APP_VERSION:下载地址
   * Range：下载范围默认下送0-
   *
   */
  public static void callInterface(Context context, HashMap<String, String> params, Callback callback) {
	  MbsLogManager.logD("#PlatformController# callInterface# 断点下载");
	    if (context == null || params == null || params.isEmpty()) {
	      throw new RequestParametersErrorException(Constants.EXCEPTION_REQUEST_PARAMS_ERROR_DETAIL, new Throwable(Constants.EXCEPTION_REQUEST_PARAMS_ERROR_DETAIL));
	    }
	    AtomicBoolean mSucceed = new AtomicBoolean();
	    String mErrorMsg = "";
	    Object object = null;
	      try {
	        object = PlatformManager.getInstance().doFetch(context, params, null);
	        mSucceed.set(true);
	      } catch (ParseDataException e) {
	        e.printStackTrace();
	        mSucceed.set(false);
	        mErrorMsg = Constants.EXCEPTION_NETWORK_DATA_ERROR;
	      } catch (NetworkException e) {
	        e.printStackTrace();
	        mSucceed.set(false);
	        mErrorMsg = Constants.EXCEPTION_NETWORK_ERROR;
	      } catch (Exception e) {
	        e.printStackTrace();
	        mSucceed.set(false);
	        mErrorMsg = Constants.EXCEPTION_EXCHANGE_FAILED;
	      } finally {
	        if (object == null) {
	          mSucceed.set(false);
	        }
	      }
	      if(mSucceed.get()){
	    	  callback.onFinished(mSucceed.get(), object);
	      }else{
	    	  callback.onFinished(mSucceed.get(), mErrorMsg);
	      }

	  }
  
  
  private static class BackgroundThread extends Thread {
    private final AtomicBoolean mCancelled = new AtomicBoolean();
    private final AtomicBoolean mSucceed = new AtomicBoolean();
    public Context mContext;
    public HashMap<String, String> mParams;
    public Callback mCallback;
    public String mErrorMsg;
    public String mChannel_url;

    public BackgroundThread(Context context, HashMap<String, String> params, String channel_url, Callback callback) {
      mContext = context;
      mParams = params;
      mCallback = callback;
      mChannel_url = channel_url;
    }

    @Override
    public void run() {
      Object object = null;
      try {
        object = PlatformManager.getInstance().doFetch(mContext, mParams, mChannel_url);
        mSucceed.set(true);
      } catch (ParseDataException e) {
        e.printStackTrace();
        mSucceed.set(false);
        mErrorMsg = Constants.EXCEPTION_NETWORK_DATA_ERROR;
      } catch (NetworkException e) {
        e.printStackTrace();
        mSucceed.set(false);
        mErrorMsg = Constants.EXCEPTION_NETWORK_ERROR;
      } catch (Exception e) {
        e.printStackTrace();
        mSucceed.set(false);
        mErrorMsg = Constants.EXCEPTION_EXCHANGE_FAILED;
      } finally {
        if (object == null) {
          mSucceed.set(false);
        }

        Result<Object> result = new Result<Object>(BackgroundThread.this, object);
        if (isSuccess()) {
          mHandler.obtainMessage(MESSAGE_POST_RESULT, result).sendToTarget();
        } else {
          mHandler.obtainMessage(MESSAGE_POST_ERROR, result).sendToTarget();
        }
      }
    }

    public boolean isCancelled() {
      return mCancelled.get();
    }

    public boolean isSuccess() {
      return mSucceed.get();
    }
  }

  private static class Result<Data> {
    public BackgroundThread mThread;
    public Data[] mData;

    public Result(BackgroundThread thread, Data... datas) {
      mThread = thread;
      mData = datas;
    }
  }

  public interface Callback {
    /**
     * 
     * 结果回调
     * 
     * @param isSuccess
     *          是否成功
     * @param map
     *          返回的结果map
     * @return void 返回类型
     */
    public void onFinished(boolean isSuccess, Object json);
  }

  /**
   * 
   * 检测当前线程是否为UI线程 校验了UI线程后，导致后台线程不能发请求。建议去掉
   * 
   * @return boolean: 返回类型
   */
  private static boolean checkUIThread() {
    long id = Thread.currentThread().getId();
    long uiId = Looper.getMainLooper().getThread().getId();
    return id == uiId;
  }

  /**
   * 
   * This isPlatformController.java class
   * 
   * @param context
   *          上下文
   * @param sqlList
   *          　交易数据库脚本
   * @param db_version
   *          　数据库版本号
   * @param localResponse
   *          　档板文件读取到内存组装成HashMap<交易码，档板文件内容>　
   * @return void: 返回类型
   */
  public static void initConfig(Context context, List<String> sqlList, int db_version, HashMap<String, String> localResponse) {
    PlatformManager.getInstance().initConfig(context, sqlList, db_version, localResponse);

  }

  // 线程池控制
  public static void runThreadControl(Runnable command) {
    PlatformManager.getInstance().runThread(command);
  }

  /**
   * 手机银行公共参数设置SKEY\MBSKEY\USERID\BRANCHID This isPlatformController.java class
   * 
   * @param para
   *          公共参数存放到HashMap下送到网络层
   * @return boolean: 返回类型
   */
  public static boolean setMBSPubPara(HashMap<String, String> para) {
    if (para == null || para.size() < 0) {
      return false;
    }
    com.ccb.mobile.platform.Constants.MBS_SKEY = para.get("SKEY") == null ? "" : para.get("SKEY");
    com.ccb.mobile.platform.Constants.MBS_MBSKEY = para.get("MBSKEY") == null ? "" : para.get("MBSKEY");
    com.ccb.mobile.platform.Constants.MBS_USERID = para.get("USERID") == null ? "" : para.get("USERID");
    com.ccb.mobile.platform.Constants.MBS_BRANCHID = para.get("BRANCHID") == null ? "" : para.get("BRANCHID");
    return true;
  }

  /**
   * 获取数据库sql参数
   * @param context
   * @param channel_url
   * @return
   */
  public static String getSYSPARAS(Context context, String channel_url) {
    return PlatformManager.getInstance().getSYSPARAS(context, channel_url);
  }
}
